@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,17 +73,12 @@ static void remmina_pref_gen_secret(void)
 	TRACE_CALL(__func__);
 	guchar s[32];
 	gint i;
-	GDateTime *gtime;
 	GKeyFile *gkeyfile;
 	g_autofree gchar *content = NULL;
 	gsize length;
 
-	gtime = g_date_time_new_now_utc();
-	srand(g_date_time_to_unix(gtime));
-	g_date_time_unref(gtime);
-
 	for (i = 0; i < 32; i++)
-		s[i] = (guchar)(rand() % 256);
+		s[i] = (guchar)(randombytes_uniform(257));
 	remmina_pref.secret = g_base64_encode(s, 32);
 
 	gkeyfile = g_key_file_new();
@@ -184,25 +179,29 @@ void remmina_pref_file_load_colors(GKeyFile *gkeyfile, RemminaColorPref *color_p
 		char **		setting;
 		char *		fallback;
 	} colors[] = {
-		{ "background", &color_pref->background, "#d5ccba" },
-		{ "cursor",	&color_pref->cursor,	 "#45373c" },
-		{ "foreground", &color_pref->foreground, "#45373c" },
-		{ "color0",	&color_pref->color0,	 "#20111b" },
-		{ "color1",	&color_pref->color1,	 "#be100e" },
-		{ "color2",	&color_pref->color2,	 "#858162" },
-		{ "color3",	&color_pref->color3,	 "#eaa549" },
-		{ "color4",	&color_pref->color4,	 "#426a79" },
-		{ "color5",	&color_pref->color5,	 "#97522c" },
-		{ "color6",	&color_pref->color6,	 "#989a9c" },
-		{ "color7",	&color_pref->color7,	 "#968c83" },
-		{ "color8",	&color_pref->color8,	 "#5e5252" },
-		{ "color9",	&color_pref->color9,	 "#be100e" },
-		{ "color10",	&color_pref->color10,	 "#858162" },
-		{ "color11",	&color_pref->color11,	 "#eaa549" },
-		{ "color12",	&color_pref->color12,	 "#426a79" },
-		{ "color13",	&color_pref->color13,	 "#97522c" },
-		{ "color14",	&color_pref->color14,	 "#989a9c" },
-		{ "color15",	&color_pref->color15,	 "#d5ccba" },
+		{ "background",		  &color_pref->background,	     "#d5ccba" },
+		{ "cursor",		  &color_pref->cursor,		     "#45373c" },
+		{ "cursor_foreground",	  &color_pref->cursor_foreground,    "#d5ccba" },
+		{ "highlight",		  &color_pref->highlight,	     "#45373c" },
+		{ "highlight_foreground", &color_pref->highlight_foreground, "#d5ccba" },
+		{ "colorBD",		  &color_pref->colorBD,		     "#45373c" },
+		{ "foreground",		  &color_pref->foreground,	     "#45373c" },
+		{ "color0",		  &color_pref->color0,		     "#20111b" },
+		{ "color1",		  &color_pref->color1,		     "#be100e" },
+		{ "color2",		  &color_pref->color2,		     "#858162" },
+		{ "color3",		  &color_pref->color3,		     "#eaa549" },
+		{ "color4",		  &color_pref->color4,		     "#426a79" },
+		{ "color5",		  &color_pref->color5,		     "#97522c" },
+		{ "color6",		  &color_pref->color6,		     "#989a9c" },
+		{ "color7",		  &color_pref->color7,		     "#968c83" },
+		{ "color8",		  &color_pref->color8,		     "#5e5252" },
+		{ "color9",		  &color_pref->color9,		     "#be100e" },
+		{ "color10",		  &color_pref->color10,		     "#858162" },
+		{ "color11",		  &color_pref->color11,		     "#eaa549" },
+		{ "color12",		  &color_pref->color12,		     "#426a79" },
+		{ "color13",		  &color_pref->color13,		     "#97522c" },
+		{ "color14",		  &color_pref->color14,		     "#989a9c" },
+		{ "color15",		  &color_pref->color15,		     "#d5ccba" },
 	};
 
 	int i;
@@ -246,6 +245,7 @@ void remmina_pref_init(void)
 
 	/* /usr/local/etc/remmina */
 	const gchar *const *dirs = g_get_system_config_dirs();
+
 	g_free(remmina_dir), remmina_dir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
 		remmina_dir = g_build_path("/", dirs[i], "remmina", NULL);
@@ -274,6 +274,7 @@ void remmina_pref_init(void)
 	remmina_keymap_file = g_strdup_printf("%s/remmina.keymap", remmina_dir);
 
 	gkeyfile = g_key_file_new();
+
 	g_key_file_load_from_file(gkeyfile, remmina_pref_file, G_KEY_FILE_NONE, NULL);
 
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "save_view_mode", NULL))
@@ -498,10 +499,25 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.disable_tray_icon = FALSE;
 
-	if (g_key_file_has_key(gkeyfile, "remmina_pref", "dark_tray_icon", NULL))
-		remmina_pref.dark_tray_icon = g_key_file_get_boolean(gkeyfile, "remmina_pref", "dark_tray_icon", NULL);
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "dark_theme", NULL))
+		remmina_pref.dark_theme = g_key_file_get_boolean(gkeyfile, "remmina_pref", "dark_theme", NULL);
 	else
-		remmina_pref.dark_tray_icon = FALSE;
+		remmina_pref.dark_theme = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "list_refresh_workaround", NULL))
+		remmina_pref.list_refresh_workaround = g_key_file_get_boolean(gkeyfile, "remmina_pref", "list_refresh_workaround", NULL);
+	else
+		remmina_pref.list_refresh_workaround = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "grab_color_switch", NULL))
+		remmina_pref.grab_color_switch = g_key_file_get_boolean(gkeyfile, "remmina_pref", "grab_color_switch", NULL);
+	else
+		remmina_pref.grab_color_switch = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "grab_color", NULL))
+		remmina_pref.grab_color = g_key_file_get_string(gkeyfile, "remmina_pref", "grab_color", NULL);
+	else
+		remmina_pref.grab_color = "#00ff00";
 
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "recent_maximum", NULL))
 		remmina_pref.recent_maximum = g_key_file_get_integer(gkeyfile, "remmina_pref", "recent_maximum", NULL);
@@ -567,6 +583,11 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.shortcutkey_viewonly = GDK_KEY_m;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "shortcutkey_multimon", NULL))
+		remmina_pref.shortcutkey_multimon = g_key_file_get_integer(gkeyfile, "remmina_pref", "shortcutkey_multimon", NULL);
+	else
+		remmina_pref.shortcutkey_multimon = GDK_KEY_Page_Up;
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "shortcutkey_grab", NULL))
 		remmina_pref.shortcutkey_grab = g_key_file_get_integer(gkeyfile, "remmina_pref", "shortcutkey_grab", NULL);
 	else
@@ -652,36 +673,13 @@ void remmina_pref_init(void)
 
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "vte_shortcutkey_search_text", NULL))
 		remmina_pref.vte_shortcutkey_search_text = g_key_file_get_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_search_text",
-										    NULL);
+										  NULL);
 	else
 		remmina_pref.vte_shortcutkey_search_text = GDK_KEY_g;
 
 
-	/* If we have a color scheme file, we switch to it, GIO will merge it in the
-	 * remmina.pref file */
-	if (g_file_test(remmina_colors_file, G_FILE_TEST_IS_REGULAR)) {
-		gkeyfile = g_key_file_new();
-		g_key_file_load_from_file(gkeyfile, remmina_colors_file, G_KEY_FILE_NONE, NULL);
-		g_remove(remmina_colors_file);
-	}
-
 	remmina_pref_file_load_colors(gkeyfile, &remmina_pref.color_pref);
 
-	if (g_key_file_has_key(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", NULL))
-		remmina_pref.periodic_usage_stats_permitted = g_key_file_get_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", NULL);
-	else
-		remmina_pref.periodic_usage_stats_permitted = FALSE;
-
-	if (g_key_file_has_key(gkeyfile, "usage_stats", "periodic_usage_stats_last_sent", NULL))
-		remmina_pref.periodic_usage_stats_last_sent = g_key_file_get_int64(gkeyfile, "usage_stats", "periodic_usage_stats_last_sent", NULL);
-	else
-		remmina_pref.periodic_usage_stats_last_sent = 0;
-
-	if (g_key_file_has_key(gkeyfile, "usage_stats", "periodic_usage_stats_uuid_prefix", NULL))
-		remmina_pref.periodic_usage_stats_uuid_prefix = g_key_file_get_string(gkeyfile, "usage_stats", "periodic_usage_stats_uuid_prefix", NULL);
-	else
-		remmina_pref.periodic_usage_stats_uuid_prefix = NULL;
-
 	/** RMNEWS_ENABLE_NEWS is equal to 0 (FALSE) when compiled with -DWiTH_NEWS=OFF,
 	 * otherwise is value is 1 (TRUE), that is the default value
 	 */
@@ -693,10 +691,13 @@ void remmina_pref_init(void)
 		remmina_pref.periodic_news_permitted = RMNEWS_ENABLE_NEWS;
 
 
-	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_last_get", NULL))
+	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_last_get", NULL)) {
 		remmina_pref.periodic_rmnews_last_get = g_key_file_get_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", NULL);
-	else
+		g_debug("(%s) - periodic_rmnews_last_get set to %ld", __func__, remmina_pref.periodic_rmnews_last_get);
+	} else {
 		remmina_pref.periodic_rmnews_last_get = 0;
+		g_debug("(%s) - periodic_rmnews_last_get set to 0", __func__);
+	}
 
 	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_get_count", NULL))
 		remmina_pref.periodic_rmnews_get_count = g_key_file_get_int64(gkeyfile, "remmina_news", "periodic_rmnews_get_count", NULL);
@@ -708,6 +709,13 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.periodic_rmnews_uuid_prefix = NULL;
 
+	/* If we have a color scheme file, we switch to it, GIO will merge it in the
+	 * remmina.pref file */
+	if (g_file_test(remmina_colors_file, G_FILE_TEST_IS_REGULAR)) {
+		g_key_file_load_from_file(gkeyfile, remmina_colors_file, G_KEY_FILE_NONE, NULL);
+		g_remove(remmina_colors_file);
+	}
+
 	/* Default settings */
 	if (!g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
 		g_key_file_set_string(gkeyfile, "remmina", "name", "");
@@ -800,7 +808,7 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "applet_hide_count", remmina_pref.applet_hide_count);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "applet_enable_avahi", remmina_pref.applet_enable_avahi);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "disable_tray_icon", remmina_pref.disable_tray_icon);
-	g_key_file_set_boolean(gkeyfile, "remmina_pref", "dark_tray_icon", remmina_pref.dark_tray_icon);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "dark_theme", remmina_pref.dark_theme);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "recent_maximum", remmina_pref.recent_maximum);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "default_mode", remmina_pref.default_mode);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "tab_mode", remmina_pref.tab_mode);
@@ -813,16 +821,29 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_prevtab", remmina_pref.shortcutkey_prevtab);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_scale", remmina_pref.shortcutkey_scale);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_grab", remmina_pref.shortcutkey_grab);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_multimon", remmina_pref.shortcutkey_multimon);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_viewonly", remmina_pref.shortcutkey_viewonly);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_screenshot", remmina_pref.shortcutkey_screenshot);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_minimize", remmina_pref.shortcutkey_minimize);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_disconnect", remmina_pref.shortcutkey_disconnect);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_toolbar", remmina_pref.shortcutkey_toolbar);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_copy", remmina_pref.vte_shortcutkey_copy);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_paste", remmina_pref.vte_shortcutkey_paste);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_select_all", remmina_pref.vte_shortcutkey_select_all);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_increase_font", remmina_pref.vte_shortcutkey_increase_font);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_decrease_font", remmina_pref.vte_shortcutkey_decrease_font);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_search_text", remmina_pref.vte_shortcutkey_search_text);
 	g_key_file_set_string(gkeyfile, "remmina_pref", "vte_font", remmina_pref.vte_font ? remmina_pref.vte_font : "");
+	g_key_file_set_string(gkeyfile, "remmina_pref", "grab_color", remmina_pref.grab_color ? remmina_pref.grab_color : "");
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "grab_color_switch", remmina_pref.grab_color_switch);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "vte_allow_bold_text", remmina_pref.vte_allow_bold_text);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "vte_lines", remmina_pref.vte_lines);
 	g_key_file_set_string(gkeyfile, "ssh_colors", "background", remmina_pref.color_pref.background ? remmina_pref.color_pref.background : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "cursor", remmina_pref.color_pref.cursor ? remmina_pref.color_pref.cursor : "");
+	g_key_file_set_string(gkeyfile, "ssh_colors", "cursor_foreground", remmina_pref.color_pref.cursor_foreground ? remmina_pref.color_pref.cursor_foreground : "");
+	g_key_file_set_string(gkeyfile, "ssh_colors", "highlight", remmina_pref.color_pref.highlight ? remmina_pref.color_pref.highlight : "");
+	g_key_file_set_string(gkeyfile, "ssh_colors", "highlight_foreground", remmina_pref.color_pref.highlight_foreground ? remmina_pref.color_pref.highlight_foreground : "");
+	g_key_file_set_string(gkeyfile, "ssh_colors", "colorBD", remmina_pref.color_pref.colorBD ? remmina_pref.color_pref.colorBD : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "foreground", remmina_pref.color_pref.foreground ? remmina_pref.color_pref.foreground : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color0", remmina_pref.color_pref.color0 ? remmina_pref.color_pref.color0 : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color1", remmina_pref.color_pref.color1 ? remmina_pref.color_pref.color1 : "");
@@ -841,11 +862,8 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color14", remmina_pref.color_pref.color14 ? remmina_pref.color_pref.color14 : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color15", remmina_pref.color_pref.color15 ? remmina_pref.color_pref.color15 : "");
 
-	g_key_file_set_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", remmina_pref.periodic_usage_stats_permitted);
-	g_key_file_set_int64(gkeyfile, "usage_stats", "periodic_usage_stats_last_sent", remmina_pref.periodic_usage_stats_last_sent);
-	g_key_file_set_string(gkeyfile, "usage_stats", "periodic_usage_stats_uuid_prefix",
-			      remmina_pref.periodic_usage_stats_uuid_prefix ? remmina_pref.periodic_usage_stats_uuid_prefix : "");
 	g_key_file_set_boolean(gkeyfile, "remmina_news", "periodic_news_permitted", remmina_pref.periodic_news_permitted);
+	g_debug("(%s) - Setting periodic_rmnews_last_get to %ld", __func__, remmina_pref.periodic_rmnews_last_get);
 	g_key_file_set_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", remmina_pref.periodic_rmnews_last_get);
 	g_key_file_set_integer(gkeyfile, "remmina_news", "periodic_rmnews_get_count", remmina_pref.periodic_rmnews_get_count);
 	g_key_file_set_string(gkeyfile, "remmina_news", "periodic_rmnews_uuid_prefix",
@@ -951,6 +969,16 @@ void remmina_pref_clear_recent(void)
 	g_key_file_free(gkeyfile);
 }
 
+guint *remmina_pref_keymap_get_table(const gchar *keymap)
+{
+	TRACE_CALL(__func__);
+	
+	if (!keymap || keymap[0] == '\0')
+		return NULL;
+	
+	return (guint *)g_hash_table_lookup(remmina_keymap_table, keymap);
+}
+
 guint remmina_pref_keymap_get_keyval(const gchar *keymap, guint keyval)
 {
 	TRACE_CALL(__func__);
