@@ -1,7 +1,7 @@
 /*
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
- * Copyright (C) 2017-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2017-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
  */
 
 #pragma once
+#include <gtk/gtk.h>
 
 /*
  * Remmina Preferences Dialog
@@ -45,25 +46,27 @@ typedef struct _RemminaPrefDialogPriv {
 
 typedef struct _RemminaPrefDialog {
 	GtkBuilder *		builder;
-	GtkDialog *		dialog;
+	GtkWidget *		dialog;
 	GtkNotebook *		notebook_preferences;
 
 	GtkFileChooser *	filechooserbutton_options_datadir_path;
 	GtkEntry *		entry_options_file_name;
 	GtkFileChooser *	filechooserbutton_options_screenshots_path;
 	GtkEntry *		entry_options_screenshot_name;
+	GtkSwitch *		switch_appearance_grab_color;
 	GtkSwitch *		switch_options_deny_screenshot_clipboard;
 	GtkSwitch *		switch_options_remember_last_view_mode;
 	GtkSwitch *		switch_security_use_master_password;
 	GtkEntry *		unlock_timeout;
 	GtkEntry *		unlock_password;
 	GtkEntry *		unlock_repassword;
+	GtkSwitch *		switch_security_trust_all;
 	GtkCheckButton *	checkbutton_options_save_settings;
 	GtkCheckButton *	checkbutton_appearance_fullscreen_on_auto;
 	GtkCheckButton *	checkbutton_appearance_show_tabs;
 	GtkCheckButton *	checkbutton_appearance_hide_toolbar;
 	GtkCheckButton *	checkbutton_appearance_hide_searchbar;
-	GtkSwitch *		switch_permit_send_stats;
+	GtkSwitch *		switch_permit_news;
 	GtkComboBox *		comboboxtext_options_double_click;
 	GtkComboBox *		comboboxtext_appearance_view_mode;
 	GtkComboBox *		comboboxtext_appearance_tab_interface;
@@ -78,13 +81,14 @@ typedef struct _RemminaPrefDialog {
 	GtkEntry *		entry_options_ssh_tcp_usrtimeout;
 	GtkEntry *		entry_options_scroll;
 	GtkEntry *		entry_options_recent_items;
+	GtkEntry *		entry_grab_color;
 	GtkButton *		button_options_recent_items_clear;
 	GtkButton *		button_options_resolutions;
 
 	GtkCheckButton *	checkbutton_applet_new_connection_on_top;
 	GtkCheckButton *	checkbutton_applet_hide_totals;
 	GtkCheckButton *	checkbutton_applet_disable_tray;
-	GtkCheckButton *	checkbutton_applet_light_tray;
+	GtkCheckButton *	checkbutton_dark_theme;
 	GtkCheckButton *	checkbutton_applet_start_in_tray;
 
 	GtkButton *		button_keyboard_host_key;
@@ -93,6 +97,7 @@ typedef struct _RemminaPrefDialog {
 	GtkButton *		button_keyboard_switch_tab_left;
 	GtkButton *		button_keyboard_switch_tab_right;
 	GtkButton *		button_keyboard_scaled;
+	GtkButton *		button_keyboard_multimon;
 	GtkButton *		button_keyboard_grab_keyboard;
 	GtkButton *		button_keyboard_screenshot;
 	GtkButton *		button_keyboard_viewonly;
@@ -113,10 +118,16 @@ typedef struct _RemminaPrefDialog {
 	GtkButton *		button_keyboard_select_all;
 	GtkButton *		button_keyboard_increase_font;
 	GtkButton *		button_keyboard_decrease_font;
+	GtkButton *		button_keyboard_search_text;
 	GtkLabel *		label_terminal_cursor_color;
+	GtkLabel *		label_terminal_bold_color;
 	GtkLabel *		label_terminal_normal_colors;
 	GtkLabel *		label_terminal_bright_colors;
 	GtkColorButton *	colorbutton_cursor;
+	GtkColorButton *	colorbutton_cursor_foreground;
+	GtkColorButton *	colorbutton_highlight;
+	GtkColorButton *	colorbutton_highlight_foreground;
+	GtkColorButton *	colorbutton_colorBD;
 	GtkColorButton *	colorbutton_color0;
 	GtkColorButton *	colorbutton_color1;
 	GtkColorButton *	colorbutton_color2;
@@ -147,9 +158,10 @@ enum {
 G_BEGIN_DECLS
 
 /* RemminaPrefDialog instance */
-GtkDialog *remmina_pref_dialog_new(gint default_tab, GtkWindow *parent);
+GtkWidget *remmina_pref_dialog_new(gint default_tab, GtkWindow *parent);
 /* Get the current PrefDialog or NULL if not initialized */
-GtkDialog *remmina_pref_dialog_get_dialog(void);
+GtkWidget *remmina_pref_dialog_get_dialog(void);
 void remmina_prefdiag_unlock_repwd_on_changed(GtkEditable *editable, RemminaPrefDialog *dialog);
+void remmina_pref_dialog_on_action_close(GSimpleAction *action, GVariant *param, gpointer data);
 
 G_END_DECLS
