@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,9 +41,9 @@
 /* Define this before stdlib.h to have posix_openpt */
 #define _XOPEN_SOURCE 600
 
+#include <errno.h>
 #define LIBSSH_STATIC 1
 #include <libssh/libssh.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -314,7 +314,8 @@ remmina_ssh_auth_pubkey(RemminaSSH *ssh, RemminaProtocolWidget *gp, RemminaFile
 	REMMINA_DEBUG("SSH certificate file: %s", ssh->certfile);
 	REMMINA_DEBUG("SSH private key file: %s", ssh->privkeyfile);
 	if (ssh->certfile != NULL) {
-		/* First we import the certificate */
+#if LIBSSH_VERSION_INT >= SSH_VERSION_INT(0, 9, 0)
+		/* First we import the private key */
 		if (ssh_pki_import_privkey_file(ssh->privkeyfile, (ssh->passphrase ? ssh->passphrase : ""),
 						NULL, NULL, &key) != SSH_OK) {
 			if (ssh->passphrase == NULL || ssh->passphrase[0] == '\0') {
@@ -327,7 +328,8 @@ remmina_ssh_auth_pubkey(RemminaSSH *ssh, RemminaProtocolWidget *gp, RemminaFile
 			return REMMINA_SSH_AUTH_AUTHFAILED_RETRY_AFTER_PROMPT;
 		}
 		REMMINA_DEBUG ("Imported private SSH key file");
-		ret = ssh_pki_import_cert_file 	(ssh->certfile, &cert ) 	;
+		/* First we import the certificate */
+		ret = ssh_pki_import_cert_file(ssh->certfile, &cert );
 		if (ret != SSH_OK) {
 			REMMINA_DEBUG ("Certificate import returned: %d", ret);
 			// TRANSLATORS: The placeholder %s is an error message
@@ -356,6 +358,9 @@ remmina_ssh_auth_pubkey(RemminaSSH *ssh, RemminaProtocolWidget *gp, RemminaFile
 			return REMMINA_SSH_AUTH_FATAL_ERROR;
 		}
 		REMMINA_DEBUG ("Authentication with a certificate file works, we can authenticate");
+#else
+		REMMINA_DEBUG ("lbssh >= 0.9.0 is required to authenticate with certificate file");
+#endif
 		/* if it goes well we authenticate (later on) with the key, not the cert*/
 	} else {
 		if (ssh->privkeyfile == NULL) {
@@ -618,7 +623,6 @@ remmina_ssh_auth(RemminaSSH *ssh, const gchar *password, RemminaProtocolWidget *
 	 *
 	 * And than test both the method and the option selected by the user
 	 */
-	ssh_userauth_none(ssh->session, NULL);
 	method = ssh_userauth_list(ssh->session, NULL);
 	REMMINA_DEBUG("Methods supported by server: %s%s%s%s%s%s%s",
 		      (method & SSH_AUTH_METHOD_NONE) ? "SSH_AUTH_METHOD_NONE " : "",
@@ -1200,7 +1204,7 @@ remmina_ssh_init_session(RemminaSSH *ssh)
 		if (rc == 0)
 			REMMINA_DEBUG("SSH_OPTIONS_IDENTITY is now %s", ssh->privkeyfile);
 		else
-			REMMINA_DEBUG("SSH_OPTIONS_IDENTITY is not set, by default the files \"identity\", \"id_dsa\" and \"id_rsa\" are used.");
+			REMMINA_DEBUG("SSH_OPTIONS_IDENTITY is not set, by default the files “identity”, “id_dsa” and “id_rsa” are used.");
 	}
 
 #ifdef SNAP_BUILD
@@ -1232,14 +1236,18 @@ remmina_ssh_init_session(RemminaSSH *ssh)
 		REMMINA_DEBUG("Setting SSH_OPTIONS_HOST to ssh->tunnel_entrance_host is 127.0.0.1,");
 		ssh_options_set(ssh->session, SSH_OPTIONS_HOST, ssh->tunnel_entrance_host);
 	}
-	rc = ssh_options_get(ssh->session, SSH_OPTIONS_USER, &parsed_config);
-	if (rc == SSH_OK) {
-		ssh->user = g_strdup(parsed_config);
-		ssh_string_free_char(parsed_config);
-	} else {
-		REMMINA_DEBUG("Parsing ssh_config for SSH_OPTIONS_USER returned an error: %s", ssh_get_error(ssh->session));
-	}
-	ssh_options_set(ssh->session, SSH_OPTIONS_USER, ssh->user);
+	if (!ssh->user || *ssh->user == 0) {
+    rc = ssh_options_get(ssh->session, SSH_OPTIONS_USER, &parsed_config);
+    if (rc == SSH_OK) {
+      if (ssh->user)
+        g_free(ssh->user);
+      ssh->user = g_strdup(parsed_config);
+      ssh_string_free_char(parsed_config);
+    } else {
+      REMMINA_DEBUG("Parsing ssh_config for SSH_OPTIONS_USER returned an error: %s", ssh_get_error(ssh->session));
+    }
+  }
+  ssh_options_set(ssh->session, SSH_OPTIONS_USER, ssh->user);
 	REMMINA_DEBUG("SSH_OPTIONS_USER is now %s", ssh->user);
 
 	/* SSH_OPTIONS_PROXYCOMMAND */
@@ -1602,7 +1610,6 @@ remmina_ssh_tunnel_new_from_file(RemminaFile *remminafile)
 	tunnel->socketbuffers = NULL;
 	tunnel->num_channels = 0;
 	tunnel->max_channels = 0;
-	tunnel->x11_channel = NULL;
 	tunnel->thread = 0;
 	tunnel->running = FALSE;
 	tunnel->server_sock = -1;
@@ -1644,13 +1651,6 @@ remmina_ssh_tunnel_close_all_channels(RemminaSSHTunnel *tunnel)
 
 	tunnel->num_channels = 0;
 	tunnel->max_channels = 0;
-
-	if (tunnel->x11_channel) {
-		ssh_channel_close(tunnel->x11_channel);
-		ssh_channel_send_eof(tunnel->x11_channel);
-		ssh_channel_free(tunnel->x11_channel);
-		tunnel->x11_channel = NULL;
-	}
 }
 
 static void
@@ -1792,47 +1792,6 @@ remmina_ssh_tunnel_main_thread_proc(gpointer data)
 		remmina_ssh_tunnel_add_channel(tunnel, channel, sock);
 		break;
 
-	case REMMINA_SSH_TUNNEL_X11:
-		if ((tunnel->x11_channel = ssh_channel_new(tunnel->ssh.session)) == NULL) {
-			// TRANSLATORS: The placeholder %s is an error message
-			remmina_ssh_set_error(REMMINA_SSH(tunnel), _("Could not create channel. %s"));
-			tunnel->thread = 0;
-			return NULL;
-		}
-		if (!remmina_public_get_xauth_cookie(tunnel->localdisplay, &ptr)) {
-			remmina_ssh_set_application_error(REMMINA_SSH(tunnel), "%s", ptr);
-			g_free(ptr);
-			tunnel->thread = 0;
-			return NULL;
-		}
-		if (ssh_channel_open_session(tunnel->x11_channel) ||
-		    ssh_channel_request_x11(tunnel->x11_channel, TRUE, NULL, ptr,
-					    gdk_x11_screen_get_screen_number(gdk_screen_get_default()))) {
-			g_free(ptr);
-			// TRANSLATORS: The placeholder %s is an error message
-			remmina_ssh_set_error(REMMINA_SSH(tunnel), _("Could not open channel. %s"));
-			tunnel->thread = 0;
-			return NULL;
-		}
-		g_free(ptr);
-		if (ssh_channel_request_exec(tunnel->x11_channel, tunnel->dest)) {
-			ptr = g_strdup_printf(_("Could not run %s on SSH server."), tunnel->dest);
-			remmina_ssh_set_error(REMMINA_SSH(tunnel), ptr);
-			g_free(ptr);
-			tunnel->thread = 0;
-			return NULL;
-		}
-
-		if (tunnel->init_func &&
-		    !(*tunnel->init_func)(tunnel, tunnel->callback_data)) {
-			if (tunnel->disconnect_func)
-				(*tunnel->disconnect_func)(tunnel, tunnel->callback_data);
-			tunnel->thread = 0;
-			return NULL;
-		}
-
-		break;
-
 	case REMMINA_SSH_TUNNEL_XPORT:
 		/* Detect the next available port starting from 6010 on the server */
 		for (i = 10; i <= MAX_X_DISPLAY_NUMBER; i++) {
@@ -1909,15 +1868,10 @@ remmina_ssh_tunnel_main_thread_proc(gpointer data)
 	/* Start the tunnel data transmission */
 	while (tunnel->running) {
 		if (tunnel->tunnel_type == REMMINA_SSH_TUNNEL_XPORT ||
-		    tunnel->tunnel_type == REMMINA_SSH_TUNNEL_X11 ||
 		    tunnel->tunnel_type == REMMINA_SSH_TUNNEL_REVERSE) {
 			if (first) {
 				first = FALSE;
-				/* Wait for a period of time for the first incoming connection */
-				if (tunnel->tunnel_type == REMMINA_SSH_TUNNEL_X11)
-					channel = ssh_channel_accept_x11(tunnel->x11_channel, 15000);
-				else
-					channel = ssh_channel_accept_forward(REMMINA_SSH(tunnel)->session, 15000, &tunnel->port);
+				channel = ssh_channel_accept_forward(REMMINA_SSH(tunnel)->session, 15000, &tunnel->port);
 				if (!channel) {
 					remmina_ssh_set_application_error(REMMINA_SSH(tunnel), _("The server did not respond."));
 					if (tunnel->disconnect_func)
@@ -1943,9 +1897,6 @@ remmina_ssh_tunnel_main_thread_proc(gpointer data)
 				       + g_date_time_difference(t1, t2) / 100000;
 				if (diff > 1) {
 					REMMINA_DEBUG("Polling tunnel channels");
-					if (tunnel->tunnel_type == REMMINA_SSH_TUNNEL_X11)
-						channel = ssh_channel_accept_x11(tunnel->x11_channel, 0);
-					else
 						channel = ssh_channel_accept_forward(REMMINA_SSH(tunnel)->session, 0, &tunnel->port);
 					if (channel == NULL)
 						t2 = t1;
@@ -1966,12 +1917,11 @@ remmina_ssh_tunnel_main_thread_proc(gpointer data)
 						close(sock);
 						sock = -1;
 					}
-				} else {
+				} else
 					sock = remmina_public_open_xdisplay(tunnel->localdisplay);
-				}
-				if (sock >= 0) {
+				if (sock >= 0)
 					remmina_ssh_tunnel_add_channel(tunnel, channel, sock);
-				} else {
+				else {
 					/* Failed to create unix socket. Will this happen? */
 					ssh_channel_close(channel);
 					ssh_channel_send_eof(channel);
@@ -2207,23 +2157,6 @@ remmina_ssh_tunnel_open(RemminaSSHTunnel *tunnel, const gchar *host, gint port,
 	return TRUE;
 }
 
-gboolean
-remmina_ssh_tunnel_x11(RemminaSSHTunnel *tunnel, const gchar *cmd)
-{
-	TRACE_CALL(__func__);
-	tunnel->tunnel_type = REMMINA_SSH_TUNNEL_X11;
-	tunnel->dest = g_strdup(cmd);
-	tunnel->running = TRUE;
-
-	if (pthread_create(&tunnel->thread, NULL, remmina_ssh_tunnel_main_thread, tunnel)) {
-		// TRANSLATORS: Do not translate pthread
-		remmina_ssh_set_application_error(REMMINA_SSH(tunnel), _("Could not start pthread."));
-		tunnel->thread = 0;
-		return FALSE;
-	}
-	return TRUE;
-}
-
 gboolean
 remmina_ssh_tunnel_xport(RemminaSSHTunnel *tunnel, gboolean bindlocalhost)
 {
@@ -2382,6 +2315,7 @@ remmina_ssh_shell_new_from_file(RemminaFile *remminafile)
 	shell->master = -1;
 	shell->slave = -1;
 	shell->exec = g_strdup(remmina_file_get_string(remminafile, "exec"));
+	shell->run_line = g_strdup(remmina_file_get_string(remminafile, "run_line"));
 
 	return shell;
 }
@@ -2418,6 +2352,9 @@ remmina_ssh_shell_thread(gpointer data)
 {
 	TRACE_CALL(__func__);
 	RemminaSSHShell *shell = (RemminaSSHShell *)data;
+	RemminaProtocolWidget *gp = (RemminaProtocolWidget *)shell->user_data;
+	RemminaFile *remminafile;
+	remminafile = remmina_protocol_widget_get_file(gp);
 	fd_set fds;
 	struct timeval timeout;
 	ssh_channel channel = NULL;
@@ -2426,6 +2363,11 @@ remmina_ssh_shell_thread(gpointer data)
 	gint buf_len;
 	gint len;
 	gint i, ret;
+	gchar *filename;
+	const gchar *dir;
+	const gchar *sshlogname;
+	FILE *fp;
+
 	//gint screen;
 
 	LOCK_SSH(shell)
@@ -2442,26 +2384,11 @@ remmina_ssh_shell_thread(gpointer data)
 
 	ssh_channel_request_pty(channel);
 
-	/* ssh_channel_request_x11 works but I do not know yet how to use the
-	 * channel. At the moment I leave the code here, commented, for future
-	 * investigations.
-	 */
-	//screen = gdk_x11_screen_get_screen_number(gdk_screen_get_default());
-	//ret = ssh_channel_request_x11 (channel, FALSE, NULL, NULL, screen);
-	/* TODO: We should have a callback that intercept an x11 request to use
-	 * ssh_channel_accept_x11 */
-	//if ( ret != SSH_OK ) {
-	//g_print ("[SSH] X11 channel error: %d\n", ret);
-	//}else {
-	//ssh_channel_accept_x11 ( channel, 50);
-	//}
-
-
 	if (shell->exec && shell->exec[0])
 		ret = ssh_channel_request_exec(channel, shell->exec);
 	else
 		ret = ssh_channel_request_shell(channel);
-	if (ret) {
+	if (ret != SSH_OK) {
 		UNLOCK_SSH(shell)
 		// TRANSLATORS: The placeholder %s is an error message
 		remmina_ssh_set_error(REMMINA_SSH(shell), _("Could not request shell. %s"));
@@ -2482,6 +2409,36 @@ remmina_ssh_shell_thread(gpointer data)
 	ch[0] = channel;
 	ch[1] = NULL;
 
+	GFile *rf = g_file_new_for_path(remminafile->filename);
+
+	if (remmina_file_get_string(remminafile, "sshlogfolder") == NULL)
+		dir = g_build_path("/", g_get_user_cache_dir(), "remmina", NULL);
+	else
+		dir = remmina_file_get_string(remminafile, "sshlogfolder");
+
+	if (remmina_file_get_string(remminafile, "sshlogname") == NULL)
+		sshlogname = g_strconcat(g_file_get_basename(rf), ".", "log", NULL);
+	else
+		sshlogname = remmina_file_get_string(remminafile, "sshlogname");
+	sshlogname = remmina_file_format_properties(remminafile, sshlogname);
+	filename = g_strconcat(dir, "/", sshlogname, NULL);
+
+	if (remmina_file_get_int (remminafile, "sshsavesession", FALSE)) {
+		REMMINA_DEBUG("Saving session log to %s", filename);
+		fp = fopen(filename, "w");
+	}
+
+	g_free(filename);
+
+	REMMINA_DEBUG("Run_line: %s", shell->run_line);
+	if (!shell->closed && shell->run_line && shell->run_line[0]) {
+		LOCK_SSH(shell)
+		//TODO: Confirm assumption - assuming null terminated gchar string
+		ssh_channel_write(channel, shell->run_line, (gint)strlen(shell->run_line));
+		ssh_channel_write(channel, "\n", (gint)1); //TODO: Test this
+		UNLOCK_SSH(shell)
+		REMMINA_DEBUG("Run_line written to channel");
+	}
 	while (!shell->closed) {
 		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
@@ -2500,12 +2457,13 @@ remmina_ssh_shell_thread(gpointer data)
 			ssh_channel_write(channel, buf, len);
 			UNLOCK_SSH(shell)
 		}
+
 		for (i = 0; i < 2; i++) {
 			LOCK_SSH(shell)
 			len = ssh_channel_poll(channel, i);
 			UNLOCK_SSH(shell)
 			if (len == SSH_ERROR || len == SSH_EOF) {
-				shell->closed = TRUE;
+				shell->closed=TRUE;
 				break;
 			}
 			if (len <= 0) continue;
@@ -2517,11 +2475,15 @@ remmina_ssh_shell_thread(gpointer data)
 			len = ssh_channel_read_nonblocking(channel, buf, len, i);
 			UNLOCK_SSH(shell)
 			if (len <= 0) {
-				shell->closed = TRUE;
+				shell->closed=TRUE;
 				break;
 			}
 			while (len > 0) {
 				ret = write(shell->slave, buf, len);
+				if (remmina_file_get_int (remminafile, "sshsavesession", FALSE)) {
+					fwrite(buf, ret, 1, fp );
+					fflush(fp);
+				}
 				if (ret <= 0) break;
 				len -= ret;
 			}
@@ -2529,6 +2491,8 @@ remmina_ssh_shell_thread(gpointer data)
 	}
 
 	LOCK_SSH(shell)
+	if (remmina_file_get_int (remminafile, "sshsavesession", FALSE))
+	fclose(fp);
 	shell->channel = NULL;
 	ssh_channel_close(channel);
 	ssh_channel_send_eof(channel);
@@ -2543,7 +2507,6 @@ remmina_ssh_shell_thread(gpointer data)
 	return NULL;
 }
 
-
 gboolean
 remmina_ssh_shell_open(RemminaSSHShell *shell, RemminaSSHExitFunc exit_callback, gpointer data)
 {
@@ -2605,6 +2568,10 @@ remmina_ssh_shell_free(RemminaSSHShell *shell)
 		g_free(shell->exec);
 		shell->exec = NULL;
 	}
+	if (shell->run_line) {
+		g_free(shell->run_line);
+		shell->run_line = NULL;
+	}
 	/* It’s not necessary to close shell->slave since the other end (vte) will close it */;
 	remmina_ssh_free(REMMINA_SSH(shell));
 }
